Rem =====================================================================
Rem Set patch release version and CPU number
Rem =====================================================================

DEFINE CPU_VERSION = '8.1.7.4.0' 
DEFINE CPU_NUMBER  = 4751906
DEFINE CPU_DESC    = 'CPUJan2006' 
SET VERIFY OFF

WHENEVER SQLERROR EXIT;

Rem =======================================================================
Rem Create table registry$history.
Rem Supress error message if table "already exists"
Rem =======================================================================

BEGIN
 execute immediate
  '
   CREATE TABLE registry$history (
   	action_time     DATE, /* time stamp */
   	action          VARCHAR2(30), /*name of action */
    	namespace       VARCHAR2(30), /* upgrade namespace */
        version         VARCHAR(30),  /*server version */
        id              NUMBER,       /* CPU or Patch ID */
	comments        VARCHAR2(255) /* comments */
)
 ';
    EXCEPTION
         WHEN OTHERS THEN
             IF sqlcode = -955 THEN NULL;
             ELSE raise;
             END IF;
END;
/ 

Rem =======================================================================
Rem Insert values for each CPU.
Rem =======================================================================

DELETE FROM registry$history WHERE id=&&CPU_NUMBER;

Rem =======================================================================
Rem Check server instance version.
Rem =======================================================================

DECLARE
  instance_version v$instance.version%type;

BEGIN

  SELECT version INTO instance_version FROM v$instance;
  IF instance_version != '&&CPU_VERSION' THEN
     RAISE_APPLICATION_ERROR(-20000,'server version does not match script');
  END IF;
END;
/


WHENEVER SQLERROR CONTINUE;

Rem =======================================================================
Rem Set event to avoid unnecessary re-compilations while
Rem catalog.sql and catproc.sql are run
Rem =======================================================================

ALTER SESSION SET EVENTS '10520 TRACE NAME CONTEXT FOREVER, LEVEL 10';

COLUMN cpu_name NEW_VALUE cpu_file NOPRINT;

Rem =======================================================================
Rem Create log file with date-time.
Rem Spool sqlplus session to logfile.
Rem =======================================================================

column cpulogfile new_value cpulogfile NOPRINT;

select 'ROLLBACK_' || name || '_' || to_char(sysdate,'DDMonYYYY_hh24_mi_ss') || '.log' as cpulogfile from v$database;


spool &cpulogfile
Rem =======================================================================
Rem Set up dbms_registry
Rem =======================================================================

@@catcr.sql

Rem =======================================================================
Rem Scripts required in all installation
Rem =======================================================================

@?/rdbms/admin/dbmsutil.sql
@?/rdbms/admin/prvtutil.plb
@?/rdbms/admin/dbmsplts.sql
@?/rdbms/admin/prvtpexp.plb
@?/rdbms/admin/prvtplts.plb
alter package DBMS_REPCAT_RGT compile;

SET SERVEROUTPUT ON;

Rem =======================================================================
Rem Count number of invalid objects and prompt user if invalid objects exists
Rem =======================================================================

DECLARE
        invalid_count NUMBER(5) := 0;
BEGIN
        select count(OBJECT_ID) into invalid_count from DBA_OBJECTS where status = 'INVALID';
        if
           invalid_count > 0
        then
           dbms_output.put_line ('No. of Invalid Objects is :' || invalid_count ) ;
           dbms_output.put_line ('Please refer to README.html to for instructions on validating these objects');
        end if;
END;
/


Prompt Logfile for the current catcpu_rollback.sql session is : &cpulogfile

SET SERVEROUTPUT OFF;

spool off;

Rem ***********************************************************************
Rem End of catcpu_rollback.sql
Rem ***********************************************************************
